-- Create the addon
Fizzle = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.0", "AceDB-2.0", "AceConsole-2.0")
Fizzle:RegisterDB("FizzleDB")
Fizzle:RegisterDefaults('profile', {
	Percent = true,
	Border = true,
	Invert = false,
	HideText = false,
	DisplayWhenFull = true,
})

function Fizzle:OnInitialize()
	-- Libraries!
	self.gratuity = AceLibrary("Gratuity-2.0")
	self.crayon = AceLibrary("Crayon-2.0")

	-- Register Chat Commands
	Fizzle:RegisterChatCommand({ "/fizzle", "/fizz" }, {
		desc = "Displays Item Quality and Durability in the character frame.",
		type = "group",
		args = {
			percent = {
				name = "Percent",
				desc = "Toggle percentage display.",
				type = "toggle",
				get = function() return self.db.profile.Percent end,
				set = function(val) self.db.profile.Percent = val self:UpdateItems() end,
			},
			border = {
				name = "Border",
				desc = "Toggle quality borders.",
				type = "toggle",
				get = function() return self.db.profile.Border end,
				set = function(val) self.db.profile.Border = val self:BorderToggle() end,
			},
			invert = {
				name = "Invert",
				desc = "Show numbers the other way around. Eg. 0% = full durability , 100 = no durability.",
				type = "toggle",
				get = function() return self.db.profile.Invert end,
				set = function(val) self.db.profile.Invert = val self:UpdateItems() end,
			},
			hidetext = {
				name = "Hide Text",
				desc = "Hide durability text.",
				type = "toggle",
				get = function() return self.db.profile.HideText end,
				set = function(val) self.db.profile.HideText = val self:UpdateItems() end,
			},
			showfull = {
				name = "Show Full",
				desc = "Show durability when full.",
				type = "toggle",
				get = function() return self.db.profile.DisplayWhenFull end,
				set = function(val) self.db.profile.DisplayWhenFull = val self:UpdateItems() end,
			}
		}
	})

	self:MakeTypeTable()
end

function Fizzle:OnEnable()
	self:RegisterEvent("UPDATE_INVENTORY_ALERTS", "UpdateItems")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "UpdateItems")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnWorldEnter")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "OnWorldLeave")
	self:Hook("CharacterFrame_OnShow")
	self:UpdateItems()
end

function Fizzle:OnDisable()
	for _,item in ipairs(self.items) do
		local str = getglobal(item.slot .. "DuraString")
		str:SetText("")
	end
	self:HideBorders()
end

function Fizzle:OnWorldEnter()
	self:RegisterEvent("UPDATE_INVENTORY_ALERTS", "UpdateItems")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "UpdateItems")
end

function Fizzle:OnWorldLeave()
	self:UnregisterEvent("UPDATE_INVENTORY_ALERTS")
	self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
end

function Fizzle:MakeTypeTable()
	-- Table of item types and slots.  Thanks Tekkub.
	self.items = self.items or {
		{name = INVTYPE_HEAD, slot = "Head"},
		{name = INVTYPE_SHOULDER, slot = "Shoulder"},
		{name = INVTYPE_CHEST, slot = "Chest"},
		{name = INVTYPE_WAIST, slot = "Waist"},
		{name = INVTYPE_LEGS, slot = "Legs"},
		{name = INVTYPE_FEET, slot = "Feet"},
		{name = INVTYPE_WRIST, slot = "Wrist"},
		{name = INVTYPE_HAND, slot = "Hands"},
		{name = INVTYPE_WEAPONMAINHAND, slot = "MainHand"},
		{name = INVTYPE_WEAPONOFFHAND, slot = "SecondaryHand"},
		{name = INVTYPE_RANGED, slot = "Ranged"},
	}
         
	-- Items without durability but with some quality, needed for border colouring.
	self.nditems = self.nditems or {
		{name = AMMOSLOT, slot = "Ammo"},
		{name = INVTYPE_NECK, slot = "Neck"},
		{name = INVTYPE_CLOAK, slot = "Back"},
		{name = INVTYPE_FINGER, slot = "Finger0"},
		{name = INVTYPE_FINGER, slot = "Finger1"},
		{name = INVTYPE_TRINKET, slot = "Trinket0"},
		{name = INVTYPE_TRINKET, slot = "Trinket1"},
		{name = RELICSLOT, slot = "Relic"},
	}
         
	-- Go through the table and create the font strings and borders for each slot.
	for _, item in ipairs(self.items) do
		local gslot = getglobal("Character" .. item.slot .. "Slot")
		if (gslot) then
			local str = gslot:CreateFontString(item.slot .. "DuraString", "OVERLAY")
			local border = gslot:CreateTexture(item.slot .. "DuraBorder", "OVERLAY")
			border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
			border:SetBlendMode("ADD")
			border:SetAlpha(0.75)
			border:SetHeight(68)
			border:SetWidth(68)
			border:SetPoint("CENTER", gslot, "CENTER", 0, 1)
			border:Hide()
			str:SetFontObject(GameFontHighlightSmallOutline)
			str:SetPoint("CENTER", gslot, "BOTTOM", 0, 8)
		end
	end

	-- Same again, but for ND items, and only creating a border
	for _, nditem in ipairs(self.nditems) do
		local ndslot = getglobal("Character" .. nditem.slot .. "Slot")
		if (ndslot) then
			local border = ndslot:CreateTexture(nditem.slot .. "DuraBorder", "OVERLAY")
			border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
			border:SetBlendMode("ADD")
			border:SetAlpha(0.75)
			border:SetHeight(68)
			border:SetWidth(68)
			border:SetPoint("CENTER", ndslot, "CENTER", 0, 1)
			border:Hide()
		end
	end
end

function Fizzle:CharacterFrame_OnShow()
	self:UpdateItems()
	-- Call the original function
	return self.hooks.CharacterFrame_OnShow.orig()
end

function Fizzle:UpdateItems()
	-- Don't update unless the charframe is open.
	-- No point updating what we can't see.
	if CharacterFrame:IsVisible() then
		-- Go and set the durability string for each slot that has an item equipped that has durability.
		-- Thanks Tekkub again for the base of this code.
		for _,item in ipairs(self.items) do
			local id, _ = GetInventorySlotInfo(item.slot .. "Slot")
			local hasItem, _, _ = self.gratuity:SetInventoryItem("player", id)

			local str = getglobal(item.slot .. "DuraString")

			local v1, v2
			if hasItem then v1, v2 = self.gratuity:FindDeformat(DURABILITY_TEMPLATE) end

			v1, v2 = tonumber(v1) or 0, tonumber(v2) or 0
			local percent = v1 / v2 * 100

			if (((v2 ~= 0) and ((percent ~= 100) or self.db.profile.DisplayWhenFull)) and not self.db.profile.HideText) then
				local text
			
				-- Colour our string depending on current durability percentage
				str:SetTextColor(self.crayon:GetThresholdColor(v1/v2))

				if (self.db.profile.Invert) then
					v1 = v2 - v1
					percent = 100 - percent
				end

				-- Are we showing the % or raw cur/max
				if (self.db.profile.Percent) then
					text = format("%d%%", percent)
				else
					text = v1.."/"..v2
				end

				str:SetText(text)
			else
				-- No durability in slot, so hide the text.
				str:SetText("")
			end
             
			--Finally, colour the borders
			if (self.db.profile.Border) then
				self:ColourBorders(id, item.slot)
			end
		end
         
		-- Colour the borders of ND items
		if (self.db.profile.Border) then
			self:ColourBordersND()
		end
	end
end

-- Border colouring split into two functions so I only need to iterate over each table once.
-- Border colouring for items with durability.
function Fizzle:ColourBorders(slotID, rawslot)
	local quality = GetInventoryItemQuality("player", slotID)
	if (quality) then
		local r, g, b, hex = GetItemQualityColor(quality)
		getglobal(rawslot .. "DuraBorder"):SetVertexColor(r, g, b)
		getglobal(rawslot .. "DuraBorder"):Show()
	else
		getglobal(rawslot .. "DuraBorder"):Hide()
	end
end

-- Border colouring for items without durability
function Fizzle:ColourBordersND()
	for _,nditem in ipairs(self.nditems) do
		if (getglobal("Character"..nditem.slot.."Slot")) then
			local slotID,_ = GetInventorySlotInfo(nditem.slot .."Slot")
			local quality = GetInventoryItemQuality("player", slotID)
			if (quality) then
				local r, g, b, hex = GetItemQualityColor(quality)
				getglobal(nditem.slot .. "DuraBorder"):SetVertexColor(r, g, b)
				getglobal(nditem.slot .. "DuraBorder"):Show()
			else
				getglobal(nditem.slot .. "DuraBorder"):Hide()
			end
		end
	end
end

-- Toggle the border colouring
function Fizzle:BorderToggle()
	if (not self.db.profile.Border) then
		self:HideBorders()
	else
		self:UpdateItems()
	end
end

-- Hide quality borders
function Fizzle:HideBorders()
	for _, item in ipairs(self.items) do
		local border = getglobal(item.slot.."DuraBorder")
		if (border) then
			border:Hide()
		end
	end
	
	for _, nditem in ipairs(self.nditems) do
		local border = getglobal(nditem.slot.."DuraBorder")
		if (border) then
			border:Hide()
		end
	end
end
